<?php

namespace ONM\RedirectLinkhandling\Handler;

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2013 David Mann <dm@onm.de>, Open New Media GmbH
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

/**
 * LinkHandler
 *
 * @package redirect_linkhandling
 * @license http://www.gnu.org/licenses/gpl.html GNU General Public License, version 3 or later
 */
class LinkHandler {

    /**
     * constant indicating that this will result in a redirect to a 404 error page
     *
     * @var int
     */
    const REDIRECT_TO_404 = 1;

    /**
     * constant indicating that this will result in a redirect to given target
     *
     * @var int
     */
    const REDIRECT_TO_TARGET = 2;

    /**
     * constant indicating that this will result in a redirect to a 404 error page and the link will be saved in db
     *
     * @var int
     */
    const NEW_URL = 3;

    /**
     * REQUEST_URI
     *
     * @var string
     */
    protected $url;

    /**
     * HTTP_HOST
     *
     * @var string
     */
    protected $domain;

    /**
     * pids of pages as array
     *
     * @var array
     */
    protected $storagePids;

    /**
     * @var \ONM\RedirectLinkhandling\Domain\Repository\RuleRepository
     */
    protected $ruleRep;

    /**
     * @var \ONM\RedirectLinkhandling\Domain\Repository\LinkRepository
     */
    protected $linkRep;

    /**
     * array with type and object of the last hit
     * types are 'Rule' and 'Link'
     *
     * @param array $lastHit
     */
    protected $lastHit;

    /**
     * objectmanager
     *
     * @param \TYPO3\CMS\Extbase\Object\ObjectManager $objectManager
     */
    protected $objectManager;

    /**
     * default constructor
     *
     * @param string $url
     */
    public function __construct($url) {
        $this->url = $url;
    }

    /**
     *
     * @param string $domain
     */
    public function setDomain($domain) {
        $this->domain = $domain;
    }

    /**
     *
     * @param array $storagePids
     */
    public function setStoragePids($storagePids) {
        $this->storagePids = $storagePids;
    }

    /**
     *
     * @return array
     */
    public function getLasthit() {
        return $this->lastHit;
    }

    /**
     * check the url for matches against rules (highest priority) and links (lower priority)
     * returns an array with the status and a target (empty string by default)
     *
     * @return array
     */
    public function getLinkHandlingArray() {
        if ($this->domain) {
            \ONM\RedirectLinkhandling\Utility\Div::initTSFE(\ONM\RedirectLinkhandling\Utility\Div::getRootPageId($domain));
        } else {
            \ONM\RedirectLinkhandling\Utility\Div::initTSFE();
        }
        if ($this->objectManager == NULL) {
            $this->objectManager = \TYPO3\CMS\Core\Utility\GeneralUtility::makeInstance('\\TYPO3\\CMS\\Extbase\\Object\\ObjectManager');
        }
        // first get all rules because they have a higher priority than the links
        if (!$this->ruleRep) {
            $this->ruleRep = $this->objectManager->get('\\ONM\\RedirectLinkhandling\\Domain\\Repository\\RuleRepository');
        }
        if ($this->storagePids) {
            $this->ruleRep->initDefaultQuerySettings($this->storagePids);
        }
        $rules = $this->ruleRep->findAll();
        if (count($rules->toArray()) > 0) {
            // handle rules stored in the storage Pages of the current rootline
            foreach ($rules as $rule) {
                $isMatch = $this->checkRule($rule, $this->url);
                if ($isMatch) {
                    if ($rule->isIgnoreonredirect()) {
                        // stop any further action and handle this as a normal 404
                        $this->lastHit = array('Rule', $rule);
                        return array(self::REDIRECT_TO_404, '');
                    }
                    // this rule is a match
                    $target = $rule->getTarget();
                    if ($target && $target != '') {
                        // redirect with 302
                        $this->lastHit = array('Rule', $rule);
                        return array(self::REDIRECT_TO_TARGET, $target);
                    }
                }
            }
        }
        // no rule did apply, so look for the link in the database
        if (!$this->linkRep) {
            $this->linkRep = $this->objectManager->get('\\ONM\\RedirectLinkhandling\\Domain\\Repository\\LinkRepository');
        }
        if ($this->storagePids) {
            $this->linkRep->initDefaultQuerySettings($this->storagePids);
        }
        $link = $this->linkRep->findByUri($this->url);
        if ($link) {
            // we have a hit
            $this->lastHit = array('Link', $link);
            if ($link->isHandled() && !$link->isHidden()) {
                // an admin marked this link as handled
                $target = $link->getTarget();
                if ($target && $target != "") {
                    // there is a target to redirect so redirect with the code 302 (permanently moved)
                    return array(self::REDIRECT_TO_TARGET, $target);
                } else {
                    // ok, this one shall be a 404
                    return array(self::REDIRECT_TO_404, '');
                }
            } else {
                // no admin handled this link or it is hidden so this is still a 404
                return array(self::REDIRECT_TO_404, '');
            }
        }

        // no matches so this is a new link
        return array(self::NEW_URL, '');
    }

    /**
     * store a new Link in database
     *
     * @return void
     */
    public function storeLink() {
        if ($this->objectManager == NULL) {
            $this->objectManager = \TYPO3\CMS\Core\Utility\GeneralUtility::makeInstance('\\TYPO3\\CMS\\Extbase\\Object\\ObjectManager');
        }
        // link is new, so create an instance of link and store it in database
        $link = $this->objectManager->get('\\ONM\\RedirectLinkhandling\\Domain\\Model\\Link');
        $link->setUri($this->url);
        if ($this->storagePids) {
            // we store the object in first page of configured pids
            $link->setPid($this->storagePids[0]);
        } else {
            // no storage Pids defined so store it in the root
            $link->setPid(0);
        }
        if (!$this->linkRep) {
            $this->linkRep = $this->objectManager->get('\\ONM\\RedirectLinkhandling\\Domain\\Repository\\LinkRepository');
        }
        try {
            $this->linkRep->add($link);
            $this->linkRep->persistAll();
        } catch (Exception $e) {
            // storing failed, logging
            \ONM\RedirectLinkhandling\Utility\Div::log($e->getMessage());
        }
    }

    /**
     * increase counter of match
     *
     * @return void
     */
    public function countHit() {
        if ($this->objectManager == NULL) {
            $this->objectManager = \TYPO3\CMS\Core\Utility\GeneralUtility::makeInstance('\\TYPO3\\CMS\\Extbase\\Object\\ObjectManager');
        }
        if (!$this->lastHit) {
            // nothing to count
            return;
        }
        switch ($this->lastHit[0]) {
            case 'Rule':
                if (!$this->ruleRep) {
                    $this->ruleRep = $this->objectManager->get('\\ONM\\RedirectLinkhandling\\Domain\\Repository\\RuleRepository');
                }
                $this->lastHit[1]->increaseHitcounter();
                $this->ruleRep->update($this->lastHit[1]);
                $this->ruleRep->persistAll();
                break;
            case 'Link':
                if (!$this->linkRep) {
                    $this->linkRep = $this->objectManager->get('\\ONM\\RedirectLinkhandling\\Domain\\Repository\\LinkRepository');
                }
                $this->lastHit[1]->increaseHitcounter();
                $this->linkRep->update($this->lastHit[1]);
                $this->linkRep->persistAll();
                break;
        }
    }

    /**
     * checks if a rule with a regular expression is a match to url
     *
     * @param \ONM\RedirectLinkhandling\Domain\Model\Rule $rule
     * @param string $complete_url
     * @return boolean
     */
    public static function checkRule($rule, $complete_url) {
        if (preg_match('/' . $rule->getExpression() . '/', $complete_url) != 0) {
            return true;
        } else {
            return false;
        }
    }

}

?>
